package myjsp.bbs;

import java.sql.*;
import java.util.*;
import javax.naming.*;
import javax.sql.*;

public class BbsManager {
	// DB 접속과 SQL처리를 위해 필요한 객체 초기화
	private DataSource ds = null;
	private Connection conn = null;
	private Statement stmt = null;
	private PreparedStatement pstmt = null;
	private ResultSet rs = null;
	private String query = "";

	// BbsManager 클래스의 생성자 정의
	// DataSource를 이용한 DBCP(Database Connection Pool) 접속
	public BbsManager() {
		try {
			Context initContext = new InitialContext();
			Context envContext = (Context) initContext.lookup("java:/comp/env");
			ds = (DataSource) envContext.lookup("jdbc/mysql/bbs");
		} catch (Exception e) {
			e.getMessage();
		} finally {
			;//System.out.println("정상적으로 접속하였습니다.");
		}
	}

	// 데이터베이스의 데이터 목록을 가져와서 Vector 객체에 담기
	public Vector<Object> getDataList(String searchField, String searchWord) throws SQLException {
		Vector<Object> DataList = new Vector<Object>();
		try {
			conn = ds.getConnection();
			stmt = conn.createStatement();
			if (searchWord == null || searchWord == "") {
				query = "select * from bbs order by pos asc";
			} else {
				query = "select * from bbs where " + searchField + " like '%"
						+ searchWord + "%' order by pos asc";
			}
			rs = stmt.executeQuery(query);
			while (rs.next()) {
				BbsData tmpData = new BbsData();

				tmpData.setNum(rs.getInt("num"));
				tmpData.setName(rs.getString("name"));
				tmpData.setEmail(rs.getString("email"));
				tmpData.setHomepage(rs.getString("homepage"));
				tmpData.setSubject(rs.getString("subject"));
				tmpData.setContent(rs.getString("content"));
				tmpData.setPos(rs.getInt("pos"));
				tmpData.setDepth(rs.getInt("depth"));
				tmpData.setRegdate(rs.getString("regdate"));
				tmpData.setPass(rs.getString("pass"));
				tmpData.setCount(rs.getInt("count"));
				DataList.addElement(tmpData);

			}
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
		return DataList;
	}

	// BbsData 객체에 저장된 데이터를  데이터베이스에 삽입하기
	public void insertData(BbsData data) throws SQLException {
		// pos 칼럼값에 1을 더한다.
		plusPos();

		try {
			conn = ds.getConnection();
			query = "insert into bbs"
					+ "(name,email,homepage,subject,content,pos,depth,regdate,pass,count,ip)"
					+ " values(?,?,?,?,?,?,?,NOW(),?,?,?)";

			pstmt = conn.prepareStatement(query);
			pstmt.setString(1, data.getName());
			pstmt.setString(2, data.getEmail());
			pstmt.setString(3, data.getHomepage());
			pstmt.setString(4, data.getSubject());
			pstmt.setString(5, data.getContent());
			pstmt.setInt(6, data.getPos());
			pstmt.setInt(7, data.getDepth());
			pstmt.setString(8, data.getPass());
			pstmt.setInt(9, data.getCount());
			pstmt.setString(10, data.getIp());
			pstmt.executeUpdate();
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
	}

	// pos 칼럼값에 1을 더한다.
	public void plusPos() throws SQLException {
		try {
			conn = ds.getConnection();
			stmt = conn.createStatement();
			query = "update bbs set pos=pos+1";
			stmt.executeUpdate(query);
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
	}

	// num 변수값의 레코드를 데이터베이스에서 가져오기
	public BbsData getData(int num) throws SQLException {
		BbsData tmpData = new BbsData();

		try {
			conn = ds.getConnection();
			query = "select * from bbs where num = ?";
			pstmt = conn.prepareStatement(query);
			pstmt.setInt(1, num);
			rs = pstmt.executeQuery();
			while (rs.next()) {
				tmpData.setNum(rs.getInt("num"));
				tmpData.setName(rs.getString("name"));
				tmpData.setEmail(rs.getString("email"));
				tmpData.setHomepage(rs.getString("homepage"));
				tmpData.setSubject(rs.getString("subject"));
				tmpData.setContent(rs.getString("content"));
				tmpData.setPos(rs.getInt("pos"));
				tmpData.setDepth(rs.getInt("depth"));
				tmpData.setRegdate(rs.getString("regdate"));
				tmpData.setPass(rs.getString("pass"));
				tmpData.setCount(rs.getInt("count"));
				tmpData.setIp(rs.getString("ip"));
			}
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
		return tmpData;
	}

	// num 변수값의 레코드에서 count 칼럼값에 1 더하기
	public void plusCount(int num) throws SQLException {
		try {
			conn = ds.getConnection();
			query = "update bbs set count=count+1 where num= ?";
			pstmt = conn.prepareStatement(query);
			pstmt.setInt(1, num);
			pstmt.executeUpdate();
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
	}

	// num 변수값의 레코드 삭제하기
	public void deleteData(int num) throws SQLException {
		try {
			conn = ds.getConnection();
			query = "delete from bbs where num = ?";
			pstmt = conn.prepareStatement(query);
			pstmt.setInt(1, num);
			pstmt.executeUpdate();
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
	}

	// BbsData 객체의 데이터를 가져와서 해당 번호의 레코드 내용을 업데이트하기
	public void updateData(BbsData data) throws SQLException {
		try {
			conn = ds.getConnection();
			query = "update bbs set "
					+ "name=?,email=?,homepage=?,subject=?,content=?"
					+ "where num=?";
			pstmt = conn.prepareStatement(query);
			pstmt.setString(1, data.getName());
			pstmt.setString(2, data.getEmail());
			pstmt.setString(3, data.getHomepage());
			pstmt.setString(4, data.getSubject());
			pstmt.setString(5, data.getContent());
			pstmt.setInt(6, data.getNum());
			pstmt.executeUpdate();
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
	}

	// 매개변수로 전달된 답변글의 BbsData 객체를 가져와서 pos 값을 얻어온다.
	// 매개변수로 넘어온 답변글의 pos 값보다 더 큰 다른 답변글들은
	// 해당 레코드의 pos 값에 1을 더한다.
	public void replyUpdate(BbsData reData) throws SQLException {
		try {
			conn = ds.getConnection();
			int pos_src = reData.getPos();
			query = "update bbs set pos = pos + 1 where pos > ?";
			pstmt = conn.prepareStatement(query);
			pstmt.setInt(1, pos_src);
			pstmt.executeUpdate();
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
	}

	// 매개변수로 받은 답변글의 BbsData 객체를 가져온다.
	// 답변글의 depth 칼럼값과 pos 칼럼값을 1 증가시키두고
	// 답변글의 내용을  데이터베이스에 삽입한다.
	public void replyInsert(BbsData reData) throws SQLException {
		try {
			conn = ds.getConnection();
			int depth_src = reData.getDepth() + 1;
			int pos_src = reData.getPos() + 1;
			query = "insert into bbs"
					+ "(name,email,homepage,subject,content,pos,depth,regdate,pass,count,ip)"
					+ " values (?,?,?,?,?,?,?,NOW(),?,?,?)";
			pstmt = conn.prepareStatement(query);
			pstmt.setString(1, reData.getName());
			pstmt.setString(2, reData.getEmail());
			pstmt.setString(3, reData.getHomepage());
			pstmt.setString(4, reData.getSubject());
			pstmt.setString(5, reData.getContent());
			pstmt.setInt(6, pos_src);
			pstmt.setInt(7, depth_src);
			pstmt.setString(8, reData.getPass());
			pstmt.setInt(9, reData.getCount());
			pstmt.setString(10, reData.getIp());
			pstmt.executeUpdate();

		} catch (Exception e) {
			System.out.println("Exception: " + e);
		} finally {
			conn.close();
		}
	}
}
