package myjsp.mail;

import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;
import javax.activation.*; // 첨부 파일 처리를 위해 필요

public class SendMail {
	// 변수를 선언한다.
	private String sender;
	private String recipient;
	private String cc;
	private String subject;
	private String contents;
	private String filename;
	private String errMsg;
	
	// 생성자에서 변수를 초기화한다.
	public SendMail() {
		sender = "";
		recipient = "";
		cc="";
		subject = "";
		contents = "";
		filename="";
		errMsg = "";
	}
	
	// 메일 발송을 위한 sendMail() 메서드
	public boolean sendMail() {
		// Properties 인스턴스 생성
		Properties props = new Properties();
		props.put("mail.transport.protocol", "smtp");
		props.put("mail.smtp.auth", "true");
		props.put("mail.smtp.host", "smtp 메일 서버명");

		try {
			// smtp 인증 사용자 정보가 있는 Authenticator 생성
			Authenticator auth = new SmtpAuthenticator();
			// 세션 인스턴스 생성(props와 auth 사용)
			Session session = Session.getDefaultInstance(props, auth);
			// session을 사용하여 MimeMessage 객체 생성
			MimeMessage msg = new MimeMessage(session);
			// 헤더를 설정한다.
			msg.addHeader("Content-Transfer-Encoding", "base64");
			msg.addHeader("Reply-To", sender);
			
			// 작성 날짜를 현재 시간으로 설정
			msg.setSentDate(new Date());
			
			// 발신자의 InternetAddress 객체 준비
			InternetAddress from = new InternetAddress(sender);
			msg.setFrom(from);
			
			// 수신자가 여러명일 때 ','를 기준으로 문자열 자르기
			if (!recipient.equals("")) {
				StringTokenizer st = new StringTokenizer(recipient, ",");
				while (st.hasMoreTokens()) {
				InternetAddress to = new InternetAddress(st.nextToken());
				msg.addRecipient(Message.RecipientType.TO, to);
				}
			}
			
			// 참조자가 여러명일 때 ','를 기준으로 문자열 자르기
			if (!cc.equals("")) {
				StringTokenizer stcc = new StringTokenizer(cc, ",");
				while (stcc.hasMoreTokens()) {
				InternetAddress tocc = new InternetAddress(stcc.nextToken());
				msg.addRecipient(Message.RecipientType.CC, tocc);
				}
			}
			// 메일 제목을 지정하면서 utf-8 캐릭터셋을 사용
			msg.setSubject(subject, "utf-8");
			
			// 메일 내용을 지정하면서 MimeMultipart, MimeBodyPart 사용
			MimeMultipart mp = new MimeMultipart();
			MimeBodyPart body = new MimeBodyPart();
			body.setContent(contents, "text/html; charset=utf-8");
			mp.addBodyPart(body);

			// 첨부파일이 있을 경우 MimeBodyPart 생성, 
			// 파일을 첨부하고 멀티파트에 MimeBodyPart 추가
			if(!filename.equals("")) {
				FileDataSource file = new FileDataSource(filename);
				MimeBodyPart attach = new MimeBodyPart();
				attach.setDataHandler(new DataHandler(file));
				// 첨부 파일 이름으로 셋팅해서 첨부
				attach.setFileName(MimeUtility.encodeWord(file.getName()));
				mp.addBodyPart(attach);
			}
			msg.setContent(mp, "text/html; charset=utf-8");
			msg.saveChanges() ;
			
			Transport tp = session.getTransport("smtp");
			tp.connect();
			tp.sendMessage(msg, msg.getAllRecipients());
			tp.close();
			
		// 예외 처리 구문
		} catch (MessagingException me) {
			me.getMessage();
			return false;
		} catch (Exception e) {
			e.getMessage();
			return false;
		}
		// 예외 또는 에러가 발생하지 않으면 성공한 것이므로 true를 리턴
		return true;
	}

	// setXxx() 메서드
	public void setSender(String sender) {this.sender = sender;}
	public void setRecipient(String recipient) {this.recipient = recipient;}
	public void setCc(String cc) {this.cc = cc;}
	public void setSubject(String subject) {this.subject = subject;}
	public void setContents(String contents) {this.contents = contents;}
	public void setFilename(String filename) {this.filename = filename;}

	// getXxx() 메서드
	public String getSender() {return sender;}
	public String getRecipient() {return recipient;}
	public String getCc() {return cc;}
	public String getSubject() {return subject;}
	public String getContents() {return contents;}
	public String getFilename() {return filename;}
	public String getErrMsg() {return errMsg;}
}

// smtp 서버 접속이 가능한 유저 아이디와 패스워드 설정
class SmtpAuthenticator extends Authenticator {
	protected PasswordAuthentication getPasswordAuthentication() {
		String id = "smtp 서버 접속 아이디";
		String pw = "패스워드";
		return new PasswordAuthentication(id, pw);
	}
}