package myjsp.mvc;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import myjsp.mvc.NullHandler;
import myjsp.mvc.CommandHandler;

public class ControllerUsingURI extends HttpServlet {
	private static final long serialVersionUID = 1L;
	
	// <커맨드, 핸들러 인스턴스> 매핑 정보를 저장한다.
	private Map<String, Object> commandHandlerMap = new java.util.HashMap<String, Object>();

	public void init(ServletConfig config) throws ServletException {
		// 매핑 정보를 저장하고 있는 설정 파일의 경로를 web.xml의 초기화 파라미터로부터 구한다.
		String conf = config.getInitParameter("configProp");
		// 매핑 정보를 저장할 Properties 객체를 생성한다.
		Properties prop = new Properties();
		// 설정 파일로부터 매핑 정보를 읽어와서 Properties 객체에 저장한다.
		FileInputStream fis = null;
		try {
			fis = new FileInputStream(conf);
			prop.load(fis);
		} catch (IOException e) {
			throw new ServletException(e);
		} finally {
			if (fis != null)
				try {
					fis.close();
				} catch (IOException ex) {
				}
		}
		Iterator<?> keyIter = prop.keySet().iterator();
		// Properties 객체에 저장된 매핑 정보를 이용하여 
		// <명령어, 핸들러 인스턴스>의 매핑 정보를 commandHandlerMap에 저장한다.
		while (keyIter.hasNext()) {
			String command = (String) keyIter.next();
			String handlerClassName = prop.getProperty(command);
			try {
				Class<?> handlerClass = Class.forName(handlerClassName);
				Object handlerInstance = handlerClass.newInstance();
				commandHandlerMap.put(command, handlerInstance);
			} catch (ClassNotFoundException e) {
				throw new ServletException(e);
			} catch (InstantiationException e) {
				throw new ServletException(e);
			} catch (IllegalAccessException e) {
				throw new ServletException(e);
			}
		}
	}
	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		process(request, response);
	}
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		process(request, response);
	}
	private void process(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		// 클라이언트의 요청(URI)을 분석한다.
		String command = request.getRequestURI();
		if(command.indexOf(request.getContextPath()) == 0) {
		command = command.substring(request.getContextPath().length());
		}
		// 요청 처리시 사용될 명령어 핸들러 인스턴스를 commandHandlerMap으로부터 얻어온다.
		CommandHandler handler = (CommandHandler) commandHandlerMap.get(command);
		// 명령어에 해당하는 핸들러 인스턴스가 존재하지 않으면 NullHandler() 메서드를 사용한다.
		if (handler == null) {
			handler = new NullHandler();
		}
		String view = null;
		try {
			// 얻어온 핸들러 인스턴스의 process() 메서드를 호출하여 
			// 요청을 처리하고 결과로 보여줄 페이지의 URI를 리턴값으로 전달받는다.
			// 핸들러 인스턴스인 handler의 process() 메서드는 클라이언트의 요청에 맞게 처리한 다음, 
			// 뷰 페이지에 보여 줄 결과값을 request나 session의 속성에 저장해야한다.
			view = handler.process(request, response);
		} catch (Throwable e) {
			throw new ServletException(e);
		}
		
		// 핸들러 인스턴스가 리턴한 뷰 페이지로 이동한다.
		RequestDispatcher dispatcher = request.getRequestDispatcher(view);
		dispatcher.forward(request, response);
	}
}